@@ -1,9 +1,37 @@
-import argparse
-import numpy as np
-import csv
-import pandas as pd
+# Copyright 2020-2023, Université de technologie de Compiègne, France,
+#                      Gabriel Frisch <gabriel.frisch@hds.utc.fr>
+# Copyright 2022-2023, Université de technologie de Compiègne, France,
+#                      Jean-Benoist Leger <jbleger@hds.utc.fr>
+#
+# Permission is hereby granted, free of charge, to any person obtaining a
+# copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+# pylint: disable=missing-module-docstring
+
 import json
+import logging
+import argparse
+
+import numpy as np
+import pandas as pd
 import scipy.sparse as sp
+
 from sparsebm import (
     SBM,
     LBM,
@@ -11,38 +39,31 @@ from sparsebm import (
     generate_LBM_dataset,
     generate_SBM_dataset,
 )
-from sparsebm.utils import reorder_rows, ARI
-import logging
+
 
 logger = logging.getLogger(__name__)
 
 try:
-    import cupy
+    import cupy  # pylint: disable=unused-import
 
     _DEFAULT_USE_GPU = True
 except ImportError:
     _DEFAULT_USE_GPU = False
 
 
-def define_parsers():
-    main = argparse.ArgumentParser(prog="sparsebm")
-    subparsers = main.add_subparsers(
+def _define_parsers():
+    main_parser = argparse.ArgumentParser(prog="sparsebm")
+    subparsers = main_parser.add_subparsers(
         help="algorithm to use", dest="subparser_name"
     )
 
-    sbm_parser = subparsers.add_parser(
-        "sbm", help="use the stochastic block model"
-    )
-    lbm_parser = subparsers.add_parser(
-        "lbm", help="use the latent block model"
-    )
+    sbm_parser = subparsers.add_parser("sbm", help="use the stochastic block model")
+    lbm_parser = subparsers.add_parser("lbm", help="use the latent block model")
     ms_parser = subparsers.add_parser(
         "modelselection", help="use the model selection with LBM or SBM"
     )
     input_grp = ms_parser.add_argument_group("mandatory arguments")
-    input_grp.add_argument(
-        "ADJACENCY_MATRIX", help="List of edges in CSV format"
-    )
+    input_grp.add_argument("ADJACENCY_MATRIX", help="List of edges in CSV format")
     input_grp.add_argument(
         "-t",
         "--type",
@@ -91,7 +112,9 @@ def define_parsers():
         "generate", help="use sparsebm to generate a data matrix"
     )
     subparsers_generate = generate_sbm_parser.add_subparsers(
-        help="model to generate data with", dest="subparsers_generate_name"
+        help="model to generate data with",
+        dest="subparsers_generate_name",
+        choices=("lbm", "sbm", "LBM", "SBM"),
     )
     sbm_generation_parser = subparsers_generate.add_parser(
         "sbm", help="use the stochastic block model to generate data"
@@ -123,9 +146,7 @@ def define_parsers():
     )
     for parser in [sbm_parser, lbm_parser]:
         input_grp = parser.add_argument_group("mandatory arguments")
-        input_grp.add_argument(
-            "ADJACENCY_MATRIX", help="List of edges in CSV format"
-        )
+        input_grp.add_argument("ADJACENCY_MATRIX", help="List of edges in CSV format")
         if parser == lbm_parser:
             input_grp.add_argument(
                 "-k1",
@@ -235,20 +256,18 @@ def define_parsers():
             type=bool,
         )
 
-    return main
+    return main_parser
 
 
-def graph_from_csv(file, type, sep=","):
+def _graph_from_csv(file_, type_, sep=","):
     try:
-        pda = pd.read_csv(file, sep=sep, header=None)
+        pda = pd.read_csv(file_, sep=sep, header=None)
 
         npa = pda[[0, 1]].to_numpy()
-        if type == "sbm":
+        if type_ == "sbm":
             node_i_from = np.unique(npa)
             node_i_to = np.arange(node_i_from.size)
-            i_mapping = {
-                f: t for f, t in np.stack((node_i_from, node_i_to), 1)
-            }
+            i_mapping = dict(np.stack((node_i_from, node_i_to), 1))
             rows = pda[0].map(i_mapping)
             cols = pda[1].map(i_mapping)
             graph = sp.coo_matrix(
@@ -256,42 +275,36 @@ def graph_from_csv(file, type, sep=","):
                 shape=(node_i_from.size, node_i_from.size),
             )
             return graph, i_mapping, None
-        else:
-            node_i_from = np.unique(npa[:, 0])
-            node_i_to = np.arange(node_i_from.size)
-            i_mapping = {
-                f: t for f, t in np.stack((node_i_from, node_i_to), 1)
-            }
-            rows = pda[0].map(i_mapping)
-            node_j_from = np.unique(npa[:, 1])
-            node_j_to = np.arange(node_j_from.size)
-            j_mapping = {
-                f: t for f, t in np.stack((node_j_from, node_j_to), 1)
-            }
-            cols = pda[1].map(j_mapping)
-            graph = sp.coo_matrix(
-                (np.ones(npa.shape[0]), (rows, cols)),
-                shape=(node_i_from.size, node_j_from.size),
-            )
-            return graph, i_mapping, j_mapping
-    except Exception as e:
-        logger.error(e)
-        raise e
+        node_i_from = np.unique(npa[:, 0])
+        node_i_to = np.arange(node_i_from.size)
+        i_mapping = dict(np.stack((node_i_from, node_i_to), 1))
+        rows = pda[0].map(i_mapping)
+        node_j_from = np.unique(npa[:, 1])
+        node_j_to = np.arange(node_j_from.size)
+        j_mapping = dict(np.stack((node_j_from, node_j_to), 1))
+        cols = pda[1].map(j_mapping)
+        graph = sp.coo_matrix(
+            (np.ones(npa.shape[0]), (rows, cols)),
+            shape=(node_i_from.size, node_j_from.size),
+        )
+        return graph, i_mapping, j_mapping
+    except Exception as expt:
+        logger.error(expt)
+        raise expt
 
 
-def str2bool(v):
-    if isinstance(v, bool):
-        return v
-    if v.lower() in ("yes", "true", "t", "y", "1"):
+def _str2bool(value):
+    if isinstance(value, bool):
+        return value
+    if value.lower() in {"yes", "true", "t", "y", "1"}:
         return True
-    elif v.lower() in ("no", "false", "f", "n", "0"):
+    if value.lower() in {"no", "false", "f", "n", "0"}:
         return False
-    else:
-        raise argparse.ArgumentTypeError("Boolean value expected.")
+    raise argparse.ArgumentTypeError("Boolean value expected.")
 
 
-def process_sbm(args):
-    graph, row_from_to, _ = graph_from_csv(
+def _process_sbm(args):  # pylint: disable=inconsistent-return-statements
+    graph, row_from_to, _ = _graph_from_csv(
         args["ADJACENCY_MATRIX"], args["subparser_name"], sep=args["sep"]
     )
     model = SBM(
@@ -305,25 +318,19 @@ def process_sbm(args):
         use_gpu=args["use_gpu"],
         gpu_index=args["gpu_index"],
     )
-    symmetric = str2bool(args["symmetric"])
-    logger.info(
-        "Runing with symmetric adjacency matrix : {}".format(symmetric)
-    )
+    symmetric = _str2bool(args["symmetric"])
+    logger.info("Runing with symmetric adjacency matrix : %s", symmetric)
     model.fit(graph, symmetric=symmetric)
 
     if not model.trained_successfully:
         logger.error("FAILED, model has not been trained successfully.")
         return None
     logger.info("Model has been trained successfully.")
-    logger.info(
-        "Value of the Integrated Completed Loglikelihood is {:.4f}".format(
-            model.get_ICL()
-        )
+    logger.info(  # pylint: disable=logging-fstring-interpolation
+        f"Value of the Integrated Completed Loglikelihood is {model.get_ICL():.4f}"
     )
     labels = model.labels
-    groups = [
-        np.argwhere(labels == q).flatten() for q in range(args["n_clusters"])
-    ]
+    groups = [np.argwhere(labels == q).flatten() for q in range(args["n_clusters"])]
     row_to_from = {v: k for k, v in row_from_to.items()}
     groups = [pd.Series(g).map(row_to_from).tolist() for g in groups]
 
@@ -334,13 +341,13 @@ def process_sbm(args):
         "node_ids_clustered": groups,
     }
 
-    with open(args["output"], "w") as outfile:
+    with open(args["output"], "w", encoding="utf8") as outfile:
         json.dump(results, outfile)
-    logger.info("Results saved in {}".format(args["output"]))
+    logger.info("Results saved in %s", args["output"])
 
 
-def process_lbm(args):
-    graph, row_from_to, col_from_to = graph_from_csv(
+def _process_lbm(args):  # pylint: disable=inconsistent-return-statements
+    graph, row_from_to, col_from_to = _graph_from_csv(
         args["ADJACENCY_MATRIX"], args["subparser_name"], sep=args["sep"]
     )
     model = LBM(
@@ -361,24 +368,20 @@ def process_lbm(args):
         logger.error("FAILED, model has not been trained successfully.")
         return None
     logger.info("Model has been trained successfully.")
-    logger.info(
-        "Value of the Integrated Completed Loglikelihood is {:.4f}".format(
-            model.get_ICL()
-        )
+    logger.info(  # pylint: disable=logging-fstring-interpolation
+        f"Value of the Integrated Completed Loglikelihood is {model.get_ICL():.4f}"
     )
 
     row_labels = model.row_labels
     row_groups = [
-        np.argwhere(row_labels == q).flatten()
-        for q in range(args["n_row_clusters"])
+        np.argwhere(row_labels == q).flatten() for q in range(args["n_row_clusters"])
     ]
     row_to_from = {v: k for k, v in row_from_to.items()}
     row_groups = [pd.Series(g).map(row_to_from).tolist() for g in row_groups]
 
     col_labels = model.column_labels
     col_groups = [
-        np.argwhere(col_labels == q).flatten()
-        for q in range(args["n_column_clusters"])
+        np.argwhere(col_labels == q).flatten() for q in range(args["n_column_clusters"])
     ]
     col_to_from = {v: k for k, v in col_from_to.items()}
     col_groups = [pd.Series(g).map(col_to_from).tolist() for g in col_groups]
@@ -386,27 +389,25 @@ def process_lbm(args):
     results = {
         "ILC": model.get_ICL(),
         "edge_probability_between_groups": model.pi_.tolist(),
-        "row_group_membership_probability": model.row_group_membership_probability.flatten().tolist(),
-        "column_group_membership_probability": model.column_group_membership_probability.flatten().tolist(),
+        "row_group_membership_probability": model.row_group_membership_probability.flatten().tolist(),  # pylint: disable=line-too-long
+        "column_group_membership_probability": model.column_group_membership_probability.flatten().tolist(),  # pylint: disable=line-too-long
         "node_type_1_ids_clustered": row_groups,
         "node_type_2_ids_clustered": col_groups,
     }
 
-    with open(args["output"], "w") as outfile:
+    with open(args["output"], "w", encoding="utf8") as outfile:
         json.dump(results, outfile)
-    logger.info("Results saved in {}".format(args["output"]))
+    logger.info("Results saved in %s", args["output"])
 
 
-def generate_sbm(args):
+def _generate_sbm(args):  # pylint: disable=too-many-locals
     if "JSON_FILE" in args:
-        with open(args["JSON_FILE"]) as f:
-            conf = json.load(f)
+        with open(args["JSON_FILE"], encoding="utf8") as fjson:
+            conf = json.load(fjson)
     else:
         conf = {}
 
-    number_of_nodes = (
-        conf["number_of_nodes"] if "number_of_nodes" in conf else None
-    )
+    number_of_nodes = conf["number_of_nodes"] if "number_of_nodes" in conf else None
     number_of_clusters = (
         conf["number_of_clusters"] if "number_of_clusters" in conf else None
     )
@@ -416,9 +417,7 @@ def generate_sbm(args):
         else None
     )
     cluster_proportions = (
-        np.array(conf["cluster_proportions"])
-        if "cluster_proportions" in conf
-        else None
+        np.array(conf["cluster_proportions"]) if "cluster_proportions" in conf else None
     )
     symmetric = conf["symmetric"] if "symmetric" in conf else False
     dataset = generate_SBM_dataset(
@@ -434,8 +433,7 @@ def generate_sbm(args):
     labels = cluster_indicator.argmax(1)
     number_of_clusters = cluster_indicator.shape[1]
     groups = [
-        np.argwhere(labels == q).flatten().tolist()
-        for q in range(number_of_clusters)
+        np.argwhere(labels == q).flatten().tolist() for q in range(number_of_clusters)
     ]
     results = {
         "node_ids_grouped": groups,
@@ -451,29 +449,25 @@ def generate_sbm(args):
 
     file_groups = "./groups.json"
     file_edges = "./edges.csv"
-    with open(file_groups, "w") as outfile:
+    with open(file_groups, "w", encoding="utf8") as outfile:
         json.dump(results, outfile)
-    logger.info("\n Groups and params saved in {}".format(file_groups))
+    logger.info("\n Groups and params saved in %s", file_groups)
     np.savetxt(file_edges, graph, delimiter=",")
-    logger.info("Edges saved in {}".format(file_edges))
+    logger.info("Edges saved in %s", file_edges)
 
 
-def generate_lbm(args):
+def _generate_lbm(args):  # pylint: disable=too-many-locals
     if "JSON_FILE" in args:
-        with open(args["JSON_FILE"]) as f:
-            conf = json.load(f)
+        with open(args["JSON_FILE"], encoding="utf8") as fjson:
+            conf = json.load(fjson)
     else:
         conf = {}
 
-    number_of_rows = (
-        conf["number_of_rows"] if "number_of_rows" in conf else None
-    )
+    number_of_rows = conf["number_of_rows"] if "number_of_rows" in conf else None
     number_of_columns = (
         conf["number_of_columns"] if "number_of_columns" in conf else None
     )
-    nb_row_clusters = (
-        conf["nb_row_clusters"] if "nb_row_clusters" in conf else None
-    )
+    nb_row_clusters = conf["nb_row_clusters"] if "nb_row_clusters" in conf else None
     nb_column_clusters = (
         conf["nb_column_clusters"] if "nb_column_clusters" in conf else None
     )
@@ -511,8 +505,7 @@ def generate_lbm(args):
     nb_row_clusters = row_cluster_indicator.shape[1]
     nb_column_clusters = column_cluster_indicator.shape[1]
     row_groups = [
-        np.argwhere(row_labels == q).flatten().tolist()
-        for q in range(nb_row_clusters)
+        np.argwhere(row_labels == q).flatten().tolist() for q in range(nb_row_clusters)
     ]
     col_groups = [
         np.argwhere(col_labels == q).flatten().tolist()
@@ -538,18 +531,20 @@ def generate_lbm(args):
 
     file_groups = "./groups.json"
     file_edges = "./edges.csv"
-    with open(file_groups, "w") as outfile:
+    with open(file_groups, "w", encoding="utf8") as outfile:
         json.dump(results, outfile)
-    logger.info("\nGroups and params saved in {}".format(file_groups))
+    logger.info("\nGroups and params saved in %s", file_groups)
     np.savetxt(file_edges, graph, delimiter=",")
-    logger.info("Edges saved in {}".format(file_edges))
+    logger.info("Edges saved in %s", file_edges)
 
 
-def process_model_selection(args):
+def _process_model_selection(  # pylint: disable=too-many-locals,inconsistent-return-statements
+    args,
+):
     if args["type"].upper() not in ["SBM", "LBM"]:
-        raise Exception("Invalid type argument. Must be 'SBM' or 'LBM'")
+        raise ValueError("Invalid type argument. Must be 'SBM' or 'LBM'")
 
-    graph, row_from_to, col_from_to = graph_from_csv(
+    graph, row_from_to, col_from_to = _graph_from_csv(
         args["ADJACENCY_MATRIX"], args["type"].lower(), sep=args["sep"]
     )
 
@@ -565,63 +560,45 @@ def process_model_selection(args):
         logger.error("FAILED, model has not been trained successfully.")
         return None
     logger.info("Model has been trained successfully.")
-    logger.info(
-        "Value of the Integrated Completed Loglikelihood is {:.4f}".format(
-            model.get_ICL()
-        )
+    logger.info(  # pylint: disable=logging-fstring-interpolation
+        f"Value of the Integrated Completed Loglikelihood is {model.get_ICL():.4f}"
     )
     if args["type"] == "lbm":
+        logger.info("The model selection picked %s row classes", model.n_row_clusters)
         logger.info(
-            "The model selection picked {} row classes".format(
-                model.n_row_clusters
-            )
-        )
-        logger.info(
-            "The model selection picked {} column classes".format(
-                model.n_column_clusters
-            )
+            "The model selection picked %s column classes", model.n_column_clusters
         )
         nb_row_clusters = model.n_row_clusters
         nb_column_clusters = model.n_column_clusters
         row_labels = model.row_labels
         row_groups = [
-            np.argwhere(row_labels == q).flatten()
-            for q in range(nb_row_clusters)
+            np.argwhere(row_labels == q).flatten() for q in range(nb_row_clusters)
         ]
         row_to_from = {v: k for k, v in row_from_to.items()}
-        row_groups = [
-            pd.Series(g).map(row_to_from).tolist() for g in row_groups
-        ]
+        row_groups = [pd.Series(g).map(row_to_from).tolist() for g in row_groups]
 
         col_labels = model.column_labels
         col_groups = [
-            np.argwhere(col_labels == q).flatten()
-            for q in range(nb_column_clusters)
+            np.argwhere(col_labels == q).flatten() for q in range(nb_column_clusters)
         ]
         col_to_from = {v: k for k, v in col_from_to.items()}
-        col_groups = [
-            pd.Series(g).map(col_to_from).tolist() for g in col_groups
-        ]
+        col_groups = [pd.Series(g).map(col_to_from).tolist() for g in col_groups]
 
         results = {
             "ILC": model.get_ICL(),
             "nb_row_clusters": nb_row_clusters,
             "nb_column_clusters": nb_column_clusters,
             "edge_probability_between_groups": model.pi_.tolist(),
-            "row_group_membership_probability": model.row_group_membership_probability.flatten().tolist(),
-            "column_group_membership_probability": model.column_group_membership_probability.flatten().tolist(),
+            "row_group_membership_probability": model.row_group_membership_probability.flatten().tolist(),  # pylint: disable=line-too-long
+            "column_group_membership_probability": model.column_group_membership_probability.flatten().tolist(),  # pylint: disable=line-too-long
             "node_type_1_ids_clustered": row_groups,
             "node_type_2_ids_clustered": col_groups,
         }
     else:
-        logger.info(
-            "The model selection picked {} classes".format(model.n_clusters)
-        )
+        logger.info("The model selection picked %s classes", model.n_clusters)
         nb_clusters = model.n_clusters
         labels = model.labels
-        groups = [
-            np.argwhere(labels == q).flatten() for q in range(nb_clusters)
-        ]
+        groups = [np.argwhere(labels == q).flatten() for q in range(nb_clusters)]
         row_to_from = {v: k for k, v in row_from_to.items()}
         groups = [pd.Series(g).map(row_to_from).tolist() for g in groups]
 
@@ -633,34 +610,34 @@ def process_model_selection(args):
             "node_ids_clustered": groups,
         }
 
-    with open(args["output"], "w") as outfile:
+    with open(args["output"], "w", encoding="utf8") as outfile:
         json.dump(results, outfile)
-    logger.info("Results saved in {}".format(args["output"]))
+    logger.info("Results saved in %s", args["output"])
 
 
-def main():
-    parsers = define_parsers()
+def main():  # pylint: disable=missing-function-docstring
+    parsers = _define_parsers()
     args = vars(parsers.parse_args())
 
     if args["subparser_name"] == "sbm":
-        process_sbm(args)
+        _process_sbm(args)
     elif args["subparser_name"] == "lbm":
-        process_lbm(args)
+        _process_lbm(args)
     elif args["subparser_name"] == "modelselection":
-        process_model_selection(args)
+        _process_model_selection(args)
     elif args["subparser_name"] == "generate":
         if (
             args["subparsers_generate_name"]
             and args["subparsers_generate_name"].upper() == "SBM"
         ):
-            generate_sbm(args)
+            _generate_sbm(args)
         elif (
             args["subparsers_generate_name"]
             and args["subparsers_generate_name"].upper() == "LBM"
         ):
-            generate_lbm(args)
+            _generate_lbm(args)
         else:
-            raise Exception(
+            raise ValueError(
                 "Specify positional argument 'sbm' or 'lbm' to generate data"
             )
 
